/**
 \brief AiFractal is a UI project code name for Fractal Generator (AiGenerator)
 \author Adrabi Abderrahim
 \date 2010-01-01
 \package AiFractals
 \page License
    \paragraph <The Artistic License>
    The intent of this document is to state the conditions under which a Package may be copied, such that the Copyright Holder maintains some semblance of artistic control over the development of the package, while giving the users of the package the right to use and distribute the Package in a more-or-less customary fashion, plus the right to make reasonable modifications.

    Definitions:
    "Package" refers to the collection of files distributed by the Copyright Holder, and derivatives of that collection of files created through textual modification.
    "Standard Version" refers to such a Package if it has not been modified, or has been modified in accordance with the wishes of the Copyright Holder.
    "Copyright Holder" is whoever is named in the copyright or copyrights for the package.
    "You" is you, if you're thinking about copying or distributing this Package.
    "Reasonable copying fee" is whatever you can justify on the basis of media cost, duplication charges, time of people involved, and so on. (You will not be required to justify it to the Copyright Holder, but only to the computing community at large as a market that must bear the fee.)
    "Freely Available" means that no fee is charged for the item itself, though there may be fees involved in handling the item. It also means that recipients of the item may redistribute it under the same conditions they received it.

    1. You may make and give away verbatim copies of the source form of the Standard Version of this Package without restriction, provided that you duplicate all of the original copyright notices and associated disclaimers.
    2. You may apply bug fixes, portability fixes and other modifications derived from the Public Domain or from the Copyright Holder. A Package modified in such a way shall still be considered the Standard Version.
    3. You may otherwise modify your copy of this Package in any way, provided that you insert a prominent notice in each changed file stating how and when you changed that file, and provided that you do at least ONE of the following:
        a) place your modifications in the Public Domain or otherwise make them Freely Available, such as by posting said modifications to Usenet or an equivalent medium, or placing the modifications on a major archive site such as ftp.uu.net, or by allowing the Copyright Holder to include your modifications in the Standard Version of the Package.
        b) use the modified Package only within your corporation or organization.
        c) rename any non-standard executables so the names do not conflict with standard executables, which must also be provided, and provide a separate manual page for each non-standard executable that clearly documents how it differs from the Standard Version.
        d) make other distribution arrangements with the Copyright Holder.

    4. You may distribute the programs of this Package in object code or executable form, provided that you do at least ONE of the following:
        a) distribute a Standard Version of the executables and library files, together with instructions (in the manual page or equivalent) on where to get the Standard Version.
        b) accompany the distribution with the machine-readable source of the Package with your modifications.
        c) accompany any non-standard executables with their corresponding Standard Version executables, giving the non-standard executables non-standard names, and clearly documenting the differences in manual pages (or equivalent), together with instructions on where to get the Standard Version.
        d) make other distribution arrangements with the Copyright Holder.

    5. You may charge a reasonable copying fee for any distribution of this Package. You may charge any fee you choose for support of this Package. You may not charge a fee for this Package itself. However, you may distribute this Package in aggregate with other (possibly commercial) programs as part of a larger (possibly commercial) software distribution provided that you do not advertise this Package as a product of your own.
    6. The scripts and library files supplied as input to or produced as output from the programs of this Package do not automatically fall under the copyright of this Package, but belong to whomever generated them, and may be sold commercially, and may be aggregated with this Package.
    7. C or perl subroutines supplied by you and linked into this Package shall not be considered part of this Package.
    8. The name of the Copyright Holder may not be used to endorse or promote products derived from this software without specific prior written permission.
    9. THIS PACKAGE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.

    The End

 */

#ifndef AICONFIG_HPP
#define AICONFIG_HPP

#include <QtGui>

#include "aiconfigtools/aiconfigtools.hpp"
#include "aiconfigfractals/aiconfigfractals.hpp"
#include "aiconfigzone/aiconfigzone.hpp"
#include "aiconfiggenerator/aiconfiggenerator.hpp"

namespace AiFractals
{
    /**
      \brief A custom class for creating a Tab tools
      \class AiConfig
      \author Adrabi Abderrahim
      \date 2010-01-01
     */
    class AiConfig : public QDialog
    {
        Q_OBJECT

    public:
        /**
          \brief contructor with optional parent
          \param parent : a widget parent ( default = 0 )
         */
        AiConfig( QWidget * parent = 0 );

        /**
          \brief destructore
         */
        ~AiConfig();

        /**
          \brief set an active tab
          \param id : set active tab by tab index
         */
        void activeTab(int id);

        /**
          \brief enable and desable child's
          \param e : true/false
         */
        void enabledChilds(bool e);

        /**
          \brief add items to combobox ( m_fractalsList ) and remove old items
          \param items : list fractals name
         */
        void addItems(const QStringList & items);

        /**
          \brief get an item string from index
          \param index : index
          \return QString
         */
        const QString item(const int index );

        /**
          \brief get a zone width
          \return double
         */
        const double getZoneWidth()   const;

        /**
          \brief set a zone width
          \param w : new zone width value
         */
        void setZoneWidth( double w );

        /**
          \brief get a zone height
          \return double
         */
        const double getZoneHeight()  const;

        /**
          \brief set a zone height
          \param h : new zone height value
         */
        void setZoneHeight( double h );

        /**
          \brief get a zone min x
          \return double
         */
        const double getZoneMinX()    const;

        /**
          \brief set a zone min x
          \param x : new zone min x value
         */
        void setZoneMinX( double x );

        /**
          \brief get a zone min y
          \return double
         */
        const double getZoneMinY() const;

        /**
          \brief set a zone min y
          \param y : new zone min y value
         */
        void setZoneMinY( double y );

        /**
          \brief get an image width
          \return int
         */
        const int getImageWidth()  const;

        /**
          \brief set an image width
          \param w : a new image width
         */
        void setImageWidth( int w );

        /**
          \brief get an image height
          \return int
         */
        const int getImageHeight() const;

        /**
          \brief set an image height
          \param h : new image height
         */
        void setImageHeight( int h );

         /**
          \brief get a number of iterations
          \return int
         */
        const int getIterations() const;

        /**
          \brief set a number of iteration
          \param i : new iteration value
         */
        void setIterations( int i );

        /**
          \brief set a threads number
          \param n : new threads number value
         */
        void setThreadsNumber( int n );

        /**
          \brief disable/uncheck zoom area
         */
        void disableZoomArea();

    protected:
        /**
          \brief event fired within dialog box is closed
          \param event : pointer
         */
        void closeEvent(QCloseEvent *event);

    private:
        /**
          \brief toolbox tab
         */
        QTabWidget * m_tabs;

    signals:
        /**
           \brief signal emitted within dialog box is closed
         */
        void closed();

        /**
          \brief *forwarded* signal for button zoom-in
         */
        void zoomIn();

        /**
          \brief *forwarded* signal for button zoom-out
         */
        void zoomOut();

        /**
          \brief *forwarded* signal for button zoom-area
          \param enable : for zoom area
         */
        void zoomArea(bool enable);

        /**
          \brief *forwarded* signal for button horizontal-mirror
         */
        void horizontalMirror();

        /**
          \brief *forwarded* signal for button vertical-mirror
         */
        void verticalMirror();

        /**
          \brief *forwarded* signal for button clear
         */
        void clear();

        /**
          \brief *forwarded* signal for dial rotation (value changed)
          \param value : current value
         */
        void rotation(int value);

        /**
            \brief *forwarded* signal emiitted for refreshing a fractals list
         */
        void refreshItems();

        /**
            \brief *forwarded* signal is sent whenever the currentIndex in the combobox changed
            \param index : current index
         */
        void currentIndexChanged ( int index );

        /**
            \brief *forwarded* signal for a fractals debugging script
         */
        void debugFractal();

        /**
          \brief *forwarded* signal for current threads number
          \param thread : threads number
         */
        void currentThreads(int thread);

        /**
          \brief *forwarded* signal for current first composition mode index
          \param val : current index value
         */
        void firstCompositionMode( int val );

        /**
          \brief *forwarded* signal for current others compisition mode index
          \brief int val : current index value
         */
        void othersCompositionMode( int val );

        /**
          \brief *forwarded* signal for running generator
         */
        void runGenerator();

        /**
          \brief *forwarded* signal for stoping generator
         */
        void stopGenerator();

    };
}
#endif // AICONFIG_HPP
