/*
Copyright 2010 Pierre SCHWARTZ

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated 
documentation files (the "Software"), to deal in the Software without restriction, including without 
limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies 
of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following 
conditions: The above copyright notice and this permission notice shall be included in all copies or 
substantial portions of the Software. THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, 
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR 
PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, 
DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF 
OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 
*/

#ifndef H__mandelbrot__
#define H__mandelbrot__

#include "fractale.h"


/*! \class mandelbrot
   * \brief Classe de description de l'ensemble de mandelbrot
   * \image html mandelbrot.png
   */
class mandelbrot : public fractale{
public:
	mandelbrot(std::complex<float>&, long);
	~mandelbrot();
	virtual std::complex<float> next(std::complex<float>&);
	bool stop();
	std::complex<float> renderingDestination(std::complex<float>&);
	inline bool isVerticalSymetric(){return true;}
	inline bool isImpair(){return false;}

	virtual mandelbrot* clone(){
		mandelbrot *b = new mandelbrot(start, maxIterations);
		return b;
	}

	static QRectF getBestWindow(){
		return QRectF(-2,-1.5,3,3);
	}

	QString getName(){
		return QString("mandel");
	}
};

#endif

